#!/usr/bin/ruby
# emoji code converter generator
#
# Ruby's License
# moriq <moriq@moriq.com>
#
def imode2id(imode)
  imode_id = nil
  imode_name = nil
  if imode =~ /^%i(\d+)%/
    imode_id = $1.to_i
  else
    imode_name = imode
  end
  return imode_id, imode_name
end

def ezweb2id(ezweb)
  ezweb_id = nil
  ezweb_name = nil
  if ezweb =~ /^%e(\d+)%/
    ezweb_id = $1.to_i
  else
    ezweb_name = ezweb
  end
  return ezweb_id, ezweb_name
end

def softbank2id(softbank)
  softbank_id = nil
  softbank_name = nil
  if softbank =~ /^%s(\d+)%/
    softbank_id = $1.to_i
  else
    softbank_name = softbank
  end
  return softbank_id, softbank_name
end

#i-modeԍ	Shift_JIS(hex)	EZweb	SoftBank	EZweb(webł̎ϊ)
#%i1%	F89F	%e44%	%s74%	
imode_ids = []
imode_id2sjis = {}
imode_id2ezweb_id = {}
imode_id2softbank_id = {}
open("emoji_i2es.txt") { |f|
  line = f.gets
  while line = f.gets
    imode, imode_sjis, ezweb, softbank = line.chomp.split(/\t/)

    imode_id, imode_name = imode2id(imode)
    ezweb_id, ezweb_name = ezweb2id(ezweb)
    softbank_id, softbank_name = softbank2id(softbank)

    if ezweb_name && softbank_name && ezweb_name != softbank_name
      $stderr.puts "name diff: ezweb[#{ezweb_name}] softbank[#{softbank_name}]"
    end

    imode_ids.push imode_id
    imode_id2sjis[imode_id] = imode_sjis
    imode_id2ezweb_id[imode_id] = ezweb_id
    imode_id2softbank_id[imode_id] = softbank_id
    # puts [imode_id, imode_sjis, ezweb_id, softbank_id, ezweb_name, softbank_name].join("\t")
  end
}

#EZwebԍ	Shift_JIS(web/hex)	i-mode	SoftBank
#%e1%	F659	%i1044%	%s282%
ezweb_id2sjis = {}
open("emoji_e2is.txt") { |f|
  line = f.gets
  while line = f.gets
    ezweb, ezweb_sjis, imode, softbank = line.chomp.split(/\t/)

    imode_id, imode_name = imode2id(imode)
    ezweb_id, ezweb_name = ezweb2id(ezweb)
    softbank_id, softbank_name = softbank2id(softbank)

    if imode_name && softbank_name && imode_name != softbank_name
      $stderr.puts "name diff: imode[#{imode_name}] softbank[#{softbank_name}]"
    end

    ezweb_id2sjis[ezweb_id] = ezweb_sjis
    # puts [ezweb_id, ezweb_sjis, imode_id, softbank_id, imode_name, softbank_name].join("\t")
  end
}

#SoftBankԍ	WebR[ḧꕔ	i-mode	EZweb
#%s1%	G!	%i140%	%e80%
softbank_id2webcode = {}
open("emoji_s2ie.txt") { |f|
  line = f.gets
  while line = f.gets
    softbank, softbank_webcode, imode, ezweb = line.chomp.split(/\t/)

    imode_id, imode_name = imode2id(imode)
    ezweb_id, ezweb_name = ezweb2id(ezweb)
    softbank_id, softbank_name = softbank2id(softbank)

    if imode_name && ezweb_name && imode_name != ezweb_name
      $stderr.puts "name diff: imode[#{imode_name}] ezweb[#{ezweb_name}]"
    end

    softbank_id2webcode[softbank_id] = softbank_webcode
    # puts [softbank_id, softbank_webcode, imode_id, ezweb_id, imode_name, ezweb_name].join("\t")
  end
}

ezweb_sjis2email_sjis = {}
open('ezweb-typed.txt') { |f|
  while line = f.gets
    ezweb_sjis, ezweb_unicode, ezweb_email_jis, ezweb_email_sjis = line.chomp.unpack("A4A4A4A4")
    ezweb_sjis2email_sjis[ezweb_sjis] = ezweb_email_sjis
  end
}

softbank_webcode2unicode = {}
open('softbank-src.txt') { |f|
  while line = f.gets
    softbank_unicode, softbank_webcode = line.chomp.split(/\t/)
    softbank_webcode2unicode[softbank_webcode] = softbank_unicode
  end
}

imode_ids.each do |imode_id|
  ezweb_id = imode_id2ezweb_id[imode_id]
  imode_sjis = imode_id2sjis[imode_id]
  ezweb_sjis = ezweb_id2sjis[ezweb_id]
  ezweb_email_sjis = ezweb_sjis2email_sjis[ezweb_sjis]
  # puts [imode_id, imode_sjis, ezweb_id, ezweb_sjis, ezweb_email_sjis].join("\t")
  puts "'#{imode_sjis}' => '#{ezweb_email_sjis}',"
end

# TODO: options
#
# 1. Hash𐶐B
#   emoji-conv-gen --from imode:sjis --to ezweb:email-sjis
#       (Zk\L) -f i:s -t e:es
#   imode, i: NTT DoCoMo i-mode
#     sjis, s	Shift_JIS 16i
#     unicode, u	Unicode
#   ezweb, e: EZweb (AU)
#     sjis, s	Shift_JIS 16i
#     unicode, u	Unicode
#     email-jis, ej	E[oJIS
#     email-sjis, es	E[oShift_JIS
#   softbank, s: SoftBank
#     unicode, u	Unicode
#     webcode, w	WebR[ḧꕔ
#
#   --module EmojiConv
#       module  default: EmojiConv
#   --method imode_sjis2ezweb_email_sjis
#       module function  default: from, to Ō܂
#   --alias sjis_i2ee
#       alias 
#
# 2. Tab؂̕\𐶐B
#   emoji-conv-gen --tab imode:id,imode:sjis,ezweb:id,ezweb:sjis,ezweb:email-sjis
#       (Zk\L) -T i:id,i:s,e:id,e:s,e:es
