#!/usr/bin/ruby
# SoftBank emoji scraper
#
# Ruby's License
# moriq <moriq@moriq.com>
#
require 'rubygems'
require 'scrapi'
require 'open-uri'
require 'cgi'

require 'optparse'

def main
  rejects = []
  @remote = false
  opts = OptionParser.new
  opts.banner = "Usage: #{File.basename($0, '.*')} [-r] [<num>]"
  opts.on('-r', '--remote', 'Get source from remote site.') { |remote|
    @remote = true
  }
  opts.on('--help', 'Prints this message and quit.') {
    puts opts.help
    exit 0
  }
  begin
    opts.parse!(ARGV)
  rescue OptionParser::ParseError => err
    $stderr.puts err.message
    exit 1
  end
  unless ARGV.size <= 1
    $stderr.puts 'wrong number of arguments'
    $stderr.puts opts.help
    exit 1
  end
  num = ARGV[0]

  @scraper = Scraper.define do
    selector :select_font, "td>font.j10"
    process "table[width=100%]>tr" do |tr|
      unicode, webcode = select_font(tr).map { |font|
        text = self.class.text(font)
        text = CGI.unescapeHTML(text)
      }
      if unicode
        if webcode =~ /\$(..)&/
          webcode_part = $1
          puts [unicode, webcode_part].join("\t")
        else
          $stderr.puts "webcode broken: unicode[#{unicode}] webcode[#{webcode}]"
        end
      end
      true
    end
  end
  @scraper.parser_options :show_warnings => true, :char_encoding => 'shiftjis'

  if num
    softbank_scrap(num)
  else
    (1..6).each { |num| softbank_scrap(num) }
  end
end

def fname(num)
  'picword_%02d.php' % num
end

def local_path(num)
  "./#{fname(num)}"
end

def local_html(num)
  File.read(local_path(num))
end

def remote_uri(num)
  'http://developers.softbankmobile.co.jp/dp/tool_dl/web/' + fname(num)
end

def source(num)
  @remote ? URI.parse(remote_uri(num)) : local_html(num)
end

def softbank_scrap(num)
  @scraper.scrape(source(num))
end

main
